export async function addNewEmojiFile(packName, file, shortcode, filename, host, token) {
  return Promise.resolve()
}

export function addressOfEmojiInPack(host, packName, name) {
  return Promise.resolve()
}

export async function createPack(host, token, packName) {
  return Promise.resolve()
}

export async function deleteEmojiFile(packName, shortcode, host, token) {
  return Promise.resolve()
}

export async function deletePack(host, token, packName) {
  return Promise.resolve()
}

export async function downloadFrom(instanceAddress, packName, as, host, token) {
  return Promise.resolve()
}

export async function fetchPack(packName, page, pageSize, host, token) {
  return Promise.resolve()
}

export async function importFromFS(host, token) {
  return Promise.resolve()
}

export async function listPacks(page, pageSize, host, token) {
  return Promise.resolve()
}

export async function listRemotePacks(instance, page, pageSize, host, token) {
  return Promise.resolve()
}

export async function reloadEmoji(host, token) {
  return Promise.resolve()
}

export async function savePackMetadata(host, token, packName, metadata) {
  return Promise.resolve()
}

export async function updateEmojiFile(packName, shortcode, newShortcode, newFilename, force, host, token) {
  return Promise.resolve()
}
